//
//  AppDelegate.m
//  Randomise List
//
//  Created by Billy on 02/12/12.
//  Copyright (c) 2012 cyneWATCH. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate
@synthesize addNew = _addNew;
@synthesize formatChoice = _formatChoice;
@synthesize answer = _answer;
@synthesize currentList = _currentList;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
  inList = [NSMutableArray array];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication {
  return YES;
}

- (IBAction)insertNew:(id)sender {
  [inList addObject:[NSString stringWithString:[_addNew stringValue]]];
  [_addNew setStringValue:@""];
  [_currentList setString:[inList componentsJoinedByString:@"\n"]];
}

- (IBAction)randomise:(id)sender {
  outList = [NSMutableArray array];
  // duplicating input array in order to keep values for later randomising
  NSMutableArray *dupInList = [NSMutableArray array];
  [dupInList addObjectsFromArray:inList];
  long inLength = [dupInList count];
  for (long i=0; i<inLength; i++) {
    int pos = arc4random() % (inLength-i);
    
    if ([_formatChoice selectedColumn] == 1) { // CSV
      [outList addObject:[NSString stringWithFormat:@"%ld; %@", i+1, dupInList[pos]]];
      defaultFileExtension = @".csv";
    } else if ([_formatChoice selectedColumn] == 2) { // HTML
      [outList addObject:[NSString stringWithFormat:@"<li>%@</li>", dupInList[pos]]];
      defaultFileExtension = @".html";
    } else if ([_formatChoice selectedColumn] == 3) { // Markdown
      [outList addObject:[NSString stringWithFormat:@"%ld. %@", i+1, dupInList[pos]]];
      defaultFileExtension = @".md";
    } else { // No formating
      [outList addObject:[NSString stringWithFormat:@"%ld. %@", i+1, dupInList[pos]]];
      defaultFileExtension = @".txt";
    }
    [dupInList removeObject:dupInList[pos]];
  }
  result = [outList componentsJoinedByString:@"\n"];
  if ([_formatChoice selectedColumn] == 2) {
    result = [[@"<ol>\n" stringByAppendingString:result] stringByAppendingString:@"\n</ol>"];
  }
  [_answer setString:result];
}

- (IBAction)openFile:(id)sender {
  openingPanel = [NSOpenPanel openPanel];
  [openingPanel setCanChooseFiles:YES];
  [openingPanel setCanChooseDirectories:NO];
  [openingPanel setAllowedFileTypes:[NSArray arrayWithObjects:@"csv", @"md", @"txt", nil]];
  [openingPanel setMessage:@"Make sure that your file only contains a list of items."];
  [openingPanel beginSheetModalForWindow:_window completionHandler:^(NSInteger result) {
    if (result == NSFileHandlingPanelOKButton) {
      NSString *fileURL = [[[openingPanel URL] absoluteString] substringFromIndex:16];
      NSString *fileContents = [NSString stringWithContentsOfFile:fileURL encoding:NSUTF8StringEncoding error:NULL];
      inList = [fileContents componentsSeparatedByString:@"\n"];
      [_currentList setString:[inList componentsJoinedByString:@"\n"]];
    }
  }];
}

- (IBAction)saveAs:(id)sender {
  savingPanel = [NSSavePanel savePanel];
  [savingPanel setAllowedFileTypes:[NSArray arrayWithObjects:@"csv", @"html", @"md", @"txt", nil]];
  [savingPanel setCanCreateDirectories:YES];
  [savingPanel setNameFieldStringValue:[@"Untitled" stringByAppendingString:defaultFileExtension]];
  [savingPanel setMessage:@"Allowed file types are .csv, .html, .md, .txt"];
  [savingPanel setExtensionHidden:NO];
  [savingPanel beginSheetModalForWindow:_window completionHandler:^(NSInteger result) {
    if (result == NSFileHandlingPanelOKButton) {
      // Stripping weird "file://localhost" from beginning of path
      NSString *fileURL = [[[savingPanel URL] absoluteString] substringFromIndex:16];
      [[NSFileManager defaultManager] createFileAtPath:fileURL contents:[@"" dataUsingEncoding:NSUnicodeStringEncoding] attributes:nil];
      oFile = [NSFileHandle fileHandleForWritingAtPath:fileURL];
      [oFile writeData:[[_answer string] dataUsingEncoding:NSUTF8StringEncoding]];
      [oFile closeFile];
    }
  }];
}

@end
